@@ -1,5 +1,5 @@
 /*=============================================================================
-    Copyright (c) 2001-2006 Joel de Guzman
+    Copyright (c) 2001-2011 Joel de Guzman
 
     Distributed under the Boost Software License, Version 1.0. (See accompanying 
     file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -7,6 +7,8 @@
 #if !defined(FUSION_END_IMPL_05062005_1226)
 #define FUSION_END_IMPL_05062005_1226
 
+#include <boost/fusion/support/config.hpp>
+
 namespace boost { namespace fusion
 {
     struct iterator_range_tag;
@@ -24,6 +26,7 @@ namespace boost { namespace fusion
             {
                 typedef typename Sequence::end_type type;
 
+                BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
                 static type
                 call(Sequence& s)
                 {
