@@ -2,6 +2,10 @@
 
 // Copyright (c) 2012-2014 Barend Gehrels, Amsterdam, the Netherlands.
 
+// This file was modified by Oracle on 2016-2017.
+// Modifications copyright (c) 2016-2017 Oracle and/or its affiliates.
+// Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
+
 // Use, modification and distribution is subject to the Boost Software License,
 // Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
 // http://www.boost.org/LICENSE_1_0.txt)
@@ -16,11 +20,14 @@
 #include <boost/core/ignore_unused.hpp>
 #include <boost/range.hpp>
 
+#include <boost/geometry/core/assert.hpp>
 #include <boost/geometry/core/coordinate_type.hpp>
 #include <boost/geometry/core/point_type.hpp>
 
+#include <boost/geometry/algorithms/comparable_distance.hpp>
 #include <boost/geometry/algorithms/covered_by.hpp>
 #include <boost/geometry/algorithms/envelope.hpp>
+#include <boost/geometry/algorithms/is_convex.hpp>
 
 #include <boost/geometry/strategies/buffer.hpp>
 
@@ -28,6 +35,7 @@
 
 #include <boost/geometry/algorithms/detail/buffer/buffered_ring.hpp>
 #include <boost/geometry/algorithms/detail/buffer/buffer_policies.hpp>
+#include <boost/geometry/algorithms/detail/overlay/cluster_info.hpp>
 #include <boost/geometry/algorithms/detail/buffer/get_piece_turns.hpp>
 #include <boost/geometry/algorithms/detail/buffer/turn_in_piece_visitor.hpp>
 #include <boost/geometry/algorithms/detail/buffer/turn_in_original_visitor.hpp>
@@ -38,6 +46,7 @@
 #include <boost/geometry/algorithms/detail/overlay/enrichment_info.hpp>
 #include <boost/geometry/algorithms/detail/overlay/enrich_intersection_points.hpp>
 #include <boost/geometry/algorithms/detail/overlay/ring_properties.hpp>
+#include <boost/geometry/algorithms/detail/overlay/select_rings.hpp>
 #include <boost/geometry/algorithms/detail/overlay/traversal_info.hpp>
 #include <boost/geometry/algorithms/detail/overlay/traverse.hpp>
 #include <boost/geometry/algorithms/detail/overlay/turn_info.hpp>
@@ -109,10 +118,13 @@ enum segment_relation_code
  */
 
 
-template <typename Ring, typename RobustPolicy>
+template <typename Ring, typename IntersectionStrategy, typename RobustPolicy>
 struct buffered_piece_collection
 {
-    typedef buffered_piece_collection<Ring, RobustPolicy> this_type;
+    typedef buffered_piece_collection
+        <
+            Ring, IntersectionStrategy, RobustPolicy
+        > this_type;
 
     typedef typename geometry::point_type<Ring>::type point_type;
     typedef typename geometry::coordinate_type<Ring>::type coordinate_type;
@@ -126,10 +138,25 @@ struct buffered_piece_collection
     typedef geometry::model::ring<robust_point_type> robust_ring_type;
     typedef geometry::model::box<robust_point_type> robust_box_type;
 
-    typedef typename strategy::side::services::default_strategy
+    typedef typename default_comparable_distance_result
         <
-            typename cs_tag<point_type>::type
-        >::type side_strategy;
+            robust_point_type
+        >::type robust_comparable_radius_type;
+
+    typedef typename IntersectionStrategy::side_strategy_type side_strategy_type;
+
+    typedef typename IntersectionStrategy::template area_strategy
+        <
+            point_type
+        >::type area_strategy_type;
+
+    typedef typename IntersectionStrategy::template area_strategy
+        <
+            robust_point_type
+        >::type robust_area_strategy_type;
+
+    typedef typename area_strategy_type::return_type area_result_type;
+    typedef typename robust_area_strategy_type::return_type robust_area_result_type;
 
     typedef typename geometry::rescale_policy_type
         <
@@ -159,7 +186,7 @@ struct buffered_piece_collection
 
     struct robust_turn
     {
-        int turn_index;
+        std::size_t turn_index;
         int operation_index;
         robust_point_type point;
         segment_identifier seg_id;
@@ -172,10 +199,10 @@ struct buffered_piece_collection
         typedef geometry::section<robust_box_type, 1> section_type;
 
         strategy::buffer::piece_type type;
-        int index;
+        signed_size_type index;
 
-        int left_index; // points to previous piece of same ring
-        int right_index; // points to next piece of same ring
+        signed_size_type left_index; // points to previous piece of same ring
+        signed_size_type right_index; // points to next piece of same ring
 
         // The next two members (1, 2) form together a complete clockwise ring
         // for each piece (with one dupped point)
@@ -183,21 +210,24 @@ struct buffered_piece_collection
 
         // 1: half, part of offsetted_rings
         segment_identifier first_seg_id;
-        int last_segment_index; // no segment-identifier - it is the same as first_seg_id
-        int offsetted_count; // part in robust_ring which is part of offsetted ring
+        signed_size_type last_segment_index; // no segment-identifier - it is the same as first_seg_id
+        signed_size_type offsetted_count; // part in robust_ring which is part of offsetted ring
 
 #if defined(BOOST_GEOMETRY_BUFFER_USE_HELPER_POINTS)
         // 2: half, not part of offsetted rings - part of robust ring
         std::vector<point_type> helper_points; // 4 points for side, 3 points for join - 0 points for flat-end
 #endif
+        bool is_flat_start;
+        bool is_flat_end;
 
+        bool is_deflated;
+        bool is_convex;
         bool is_monotonic_increasing[2]; // 0=x, 1=y
         bool is_monotonic_decreasing[2]; // 0=x, 1=y
 
         // Monotonic sections of pieces around points
         std::vector<section_type> sections;
 
-
         // Robust representations
         // 3: complete ring
         robust_ring_type robust_ring;
@@ -206,6 +236,30 @@ struct buffered_piece_collection
         robust_box_type robust_offsetted_envelope;
 
         std::vector<robust_turn> robust_turns; // Used only in insert_rescaled_piece_turns - we might use a map instead
+
+        robust_point_type robust_center;
+        robust_comparable_radius_type robust_min_comparable_radius;
+        robust_comparable_radius_type robust_max_comparable_radius;
+
+        piece()
+            : type(strategy::buffer::piece_type_unknown)
+            , index(-1)
+            , left_index(-1)
+            , right_index(-1)
+            , last_segment_index(-1)
+            , offsetted_count(-1)
+            , is_flat_start(false)
+            , is_flat_end(false)
+            , is_deflated(false)
+            , is_convex(false)
+            , robust_min_comparable_radius(0)
+            , robust_max_comparable_radius(0)
+        {
+            is_monotonic_increasing[0] = false;
+            is_monotonic_increasing[1] = false;
+            is_monotonic_decreasing[0] = false;
+            is_monotonic_decreasing[1] = false;
+        }
     };
 
     struct robust_original
@@ -226,8 +280,11 @@ struct buffered_piece_collection
         {
             geometry::envelope(m_ring, m_box);
 
-            // create monotonic sections in y-dimension
-            typedef boost::mpl::vector_c<std::size_t, 1> dimensions;
+            // create monotonic sections in x-dimension
+            // The dimension is critical because the direction is later used
+            // in the optimization for within checks using winding strategy
+            // and this strategy is scanning in x direction.
+            typedef boost::mpl::vector_c<std::size_t, 0> dimensions;
             geometry::sectionalize<false, dimensions>(m_ring,
                     detail::no_rescale_policy(), m_sections);
         }
@@ -244,7 +301,9 @@ struct buffered_piece_collection
 
     piece_vector_type m_pieces;
     turn_vector_type m_turns;
-    int m_first_piece_index;
+    signed_size_type m_first_piece_index;
+    bool m_deflate;
+    bool m_has_deflated;
 
     buffered_ring_collection<buffered_ring<Ring> > offsetted_rings; // indexed by multi_index
     std::vector<robust_original> robust_originals; // robust representation of the original(s)
@@ -257,7 +316,19 @@ struct buffered_piece_collection
     typedef geometry::sections<robust_box_type, 2> sections_type;
     sections_type monotonic_sections;
 
+    // Define the clusters, mapping cluster_id -> turns
+    typedef std::map
+        <
+            signed_size_type,
+            detail::overlay::cluster_info
+        > cluster_type;
 
+    cluster_type m_clusters;
+
+    IntersectionStrategy m_intersection_strategy;
+    side_strategy_type m_side_strategy;
+    area_strategy_type m_area_strategy;
+    robust_area_strategy_type m_robust_area_strategy;
     RobustPolicy const& m_robust_policy;
 
     struct redundant_turn
@@ -268,8 +339,15 @@ struct buffered_piece_collection
         }
     };
 
-    buffered_piece_collection(RobustPolicy const& robust_policy)
+    buffered_piece_collection(IntersectionStrategy const& intersection_strategy,
+                              RobustPolicy const& robust_policy)
         : m_first_piece_index(-1)
+        , m_deflate(false)
+        , m_has_deflated(false)
+        , m_intersection_strategy(intersection_strategy)
+        , m_side_strategy(intersection_strategy.get_side_strategy())
+        , m_area_strategy(intersection_strategy.template get_area_strategy<point_type>())
+        , m_robust_area_strategy(intersection_strategy.template get_area_strategy<robust_point_type>())
         , m_robust_policy(robust_policy)
     {}
 
@@ -427,11 +505,11 @@ struct buffered_piece_collection
         for (typename occupation_map_type::iterator it = occupation_map.begin();
             it != occupation_map.end(); ++it)
         {
-            it->second.get_left_turns(it->first, m_turns);
+            it->second.get_left_turns(it->first, m_turns, m_side_strategy);
         }
     }
 
-    inline void classify_turns(bool linear)
+    inline void classify_turns()
     {
         for (typename boost::range_iterator<turn_vector_type>::type it =
             boost::begin(m_turns); it != boost::end(m_turns); ++it)
@@ -440,16 +518,103 @@ struct buffered_piece_collection
             {
                 it->location = inside_buffer;
             }
-            if (it->count_on_original_boundary > 0 && ! linear)
+            if (it->count_on_original_boundary > 0)
             {
                 it->location = inside_buffer;
             }
+#if ! defined(BOOST_GEOMETRY_BUFFER_USE_SIDE_OF_INTERSECTION)
             if (it->count_within_near_offsetted > 0)
             {
                 // Within can have in rare cases a rounding issue. We don't discard this
                 // point, so it can be used to continue started rings in traversal. But
                 // will never start a new ring from this type of points.
-                it->selectable_start = false;
+                it->operations[0].enriched.startable = false;
+                it->operations[1].enriched.startable = false;
+            }
+#endif
+        }
+    }
+
+    struct deflate_properties
+    {
+        bool has_inflated;
+        std::size_t count;
+
+        inline deflate_properties()
+            : has_inflated(false)
+            , count(0u)
+        {}
+    };
+
+    inline void discard_turns_for_deflate()
+    {
+        // Deflate cases should have at least 3 points PER deflated original
+        // to form a correct triangle
+
+        // But if there are intersections between a deflated ring and another
+        // ring, it is all accepted
+
+        // In deflate most turns are i/u by nature, but u/u is also possible
+
+        std::map<signed_size_type, deflate_properties> properties;
+
+        for (typename boost::range_iterator<turn_vector_type const>::type it =
+            boost::begin(m_turns); it != boost::end(m_turns); ++it)
+        {
+            const buffer_turn_info_type& turn = *it;
+            if (turn.location == location_ok)
+            {
+                const buffer_turn_operation_type& op0 = turn.operations[0];
+                const buffer_turn_operation_type& op1 = turn.operations[1];
+
+                if (! m_pieces[op0.seg_id.piece_index].is_deflated
+                 || ! m_pieces[op1.seg_id.piece_index].is_deflated)
+                {
+                    properties[op0.seg_id.multi_index].has_inflated = true;
+                    properties[op1.seg_id.multi_index].has_inflated = true;
+                    continue;
+                }
+
+                // It is deflated, update counts
+                for (int i = 0; i < 2; i++)
+                {
+                    const buffer_turn_operation_type& op = turn.operations[i];
+                    if (op.operation == detail::overlay::operation_union
+                        || op.operation == detail::overlay::operation_continue)
+                    {
+                        properties[op.seg_id.multi_index].count++;
+                    }
+                }
+            }
+        }
+
+        for (typename boost::range_iterator<turn_vector_type>::type it =
+            boost::begin(m_turns); it != boost::end(m_turns); ++it)
+        {
+            buffer_turn_info_type& turn = *it;
+
+            if (turn.location == location_ok)
+            {
+                const buffer_turn_operation_type& op0 = turn.operations[0];
+                const buffer_turn_operation_type& op1 = turn.operations[1];
+                signed_size_type const multi0 = op0.seg_id.multi_index;
+                signed_size_type const multi1 = op1.seg_id.multi_index;
+
+                if (multi0 == multi1)
+                {
+                    const deflate_properties& prop =  properties[multi0];
+                    if (! prop.has_inflated && prop.count < 3)
+                    {
+                        // Property is not inflated
+                        // Not enough points, this might be caused by <float> where
+                        // detection turn-in-original failed because of numeric errors
+                        turn.location = location_discard;
+                    }
+                }
+                else
+                {
+                    // Two different (possibly deflated) rings
+                }
             }
         }
     }
@@ -463,10 +628,11 @@ struct buffered_piece_collection
         geometry::partition
             <
                 robust_box_type,
-                turn_get_box, turn_in_original_ovelaps_box,
-                original_get_box, original_ovelaps_box,
-                include_turn_policy, detail::partition::include_all_policy
-            >::apply(m_turns, robust_originals, visitor);
+                include_turn_policy,
+                detail::partition::include_all_policy
+            >::apply(m_turns, robust_originals, visitor,
+                     turn_get_box(), turn_in_original_ovelaps_box(),
+                     original_get_box(), original_ovelaps_box());
 
         bool const deflate = distance_strategy.negative();
 
@@ -478,7 +644,7 @@ struct buffered_piece_collection
             {
                 if (deflate && turn.count_in_original <= 0)
                 {
-                    // For deflate: it is not in original, discard
+                    // For deflate/negative buffers: it is not in original, discard
                     turn.location = location_discard;
                 }
                 else if (! deflate && turn.count_in_original > 0)
@@ -488,6 +654,12 @@ struct buffered_piece_collection
                 }
             }
         }
+
+        if (m_has_deflated)
+        {
+            // Either strategy was negative, or there were interior rings
+            discard_turns_for_deflate();
+        }
     }
 
     inline bool assert_indices_in_robust_rings() const
@@ -515,7 +687,7 @@ struct buffered_piece_collection
     {
         // Add rescaled turn points to corresponding pieces
         // (after this, each turn occurs twice)
-        int index = 0;
+        std::size_t index = 0;
         for (typename boost::range_iterator<turn_vector_type>::type it =
             boost::begin(m_turns); it != boost::end(m_turns); ++it, ++index)
         {
@@ -544,16 +716,16 @@ struct buffered_piece_collection
             }
         }
 
+#if ! defined(BOOST_GEOMETRY_BUFFER_USE_SIDE_OF_INTERSECTION)
         // Insert all rescaled turn-points into these rings, to form a
         // reliable integer-based ring. All turns can be compared (inside) to this
         // rings to see if they are inside.
 
-        for (typename piece_vector_type::iterator it = boost::begin(m_pieces);
-            it != boost::end(m_pieces);
-            ++it)
+        for (typename boost::range_iterator<piece_vector_type>::type
+                it = boost::begin(m_pieces); it != boost::end(m_pieces); ++it)
         {
             piece& pc = *it;
-            int piece_segment_index = pc.first_seg_id.segment_index;
+            signed_size_type piece_segment_index = pc.first_seg_id.segment_index;
             if (! pc.robust_turns.empty())
             {
                 if (pc.robust_turns.size() > 1u)
@@ -561,14 +733,14 @@ struct buffered_piece_collection
                     std::sort(pc.robust_turns.begin(), pc.robust_turns.end(), buffer_operation_less());
                 }
                 // Walk through them, in reverse to insert at right index
-                int index_offset = pc.robust_turns.size() - 1;
-                for (typename std::vector<robust_turn>::const_reverse_iterator
-                        rit = pc.robust_turns.rbegin();
-                    rit != pc.robust_turns.rend();
+                signed_size_type index_offset = static_cast<signed_size_type>(pc.robust_turns.size()) - 1;
+                for (typename boost::range_reverse_iterator<const std::vector<robust_turn> >::type
+                        rit = boost::const_rbegin(pc.robust_turns);
+                    rit != boost::const_rend(pc.robust_turns);
                     ++rit, --index_offset)
                 {
-                    int const index_in_vector = 1 + rit->seg_id.segment_index - piece_segment_index;
-                    BOOST_ASSERT
+                    signed_size_type const index_in_vector = 1 + rit->seg_id.segment_index - piece_segment_index;
+                    BOOST_GEOMETRY_ASSERT
                     (
                         index_in_vector > 0
                         && index_in_vector < pc.offsetted_count
@@ -582,7 +754,8 @@ struct buffered_piece_collection
             }
         }
 
-        BOOST_ASSERT(assert_indices_in_robust_rings());
+        BOOST_GEOMETRY_ASSERT(assert_indices_in_robust_rings());
+#endif
     }
 
     template <std::size_t Dimension>
@@ -607,6 +780,8 @@ struct buffered_piece_collection
         pc.is_monotonic_decreasing[0] = true;
         pc.is_monotonic_decreasing[1] = true;
 
+        pc.is_convex = geometry::is_convex(pc.robust_ring);
+
         if (pc.offsetted_count < 2)
         {
             return;
@@ -615,14 +790,13 @@ struct buffered_piece_collection
         typename robust_ring_type::const_iterator current = pc.robust_ring.begin();
         typename robust_ring_type::const_iterator next = current + 1;
 
-        for (int i = 1; i < pc.offsetted_count; i++)
+        for (signed_size_type i = 1; i < pc.offsetted_count; i++)
         {
             determine_monotonicity<0>(pc, *current, *next);
             determine_monotonicity<1>(pc, *current, *next);
             current = next;
             ++next;
         }
-
     }
 
     void determine_properties()
@@ -642,7 +816,7 @@ struct buffered_piece_collection
             ++it)
         {
             piece& pc = *it;
-            if (geometry::area(pc.robust_ring) < 0)
+            if (geometry::area(pc.robust_ring, m_robust_area_strategy) < 0)
             {
                 // Rings can be ccw:
                 // - in a concave piece
@@ -659,7 +833,31 @@ struct buffered_piece_collection
         geometry::sectionalize<false, dimensions>(pc.robust_ring,
                 detail::no_rescale_policy(), pc.sections);
 
-        // TODO (next phase) determine min/max radius
+        // Determine min/max radius
+        typedef geometry::model::referring_segment<robust_point_type const>
+            robust_segment_type;
+
+        typename robust_ring_type::const_iterator current = pc.robust_ring.begin();
+        typename robust_ring_type::const_iterator next = current + 1;
+
+        for (signed_size_type i = 1; i < pc.offsetted_count; i++)
+        {
+            robust_segment_type s(*current, *next);
+            robust_comparable_radius_type const d
+                = geometry::comparable_distance(pc.robust_center, s);
+
+            if (i == 1 || d < pc.robust_min_comparable_radius)
+            {
+                pc.robust_min_comparable_radius = d;
+            }
+            if (i == 1 || d > pc.robust_max_comparable_radius)
+            {
+                pc.robust_max_comparable_radius = d;
+            }
+
+            current = next;
+            ++next;
+        }
     }
 
     inline void prepare_buffered_point_pieces()
@@ -692,15 +890,17 @@ struct buffered_piece_collection
                     piece_vector_type,
                     buffered_ring_collection<buffered_ring<Ring> >,
                     turn_vector_type,
+                    IntersectionStrategy,
                     RobustPolicy
-                > visitor(m_pieces, offsetted_rings, m_turns, m_robust_policy);
+                > visitor(m_pieces, offsetted_rings, m_turns,
+                          m_intersection_strategy, m_robust_policy);
 
             geometry::partition
                 <
-                    robust_box_type,
-                    detail::section::get_section_box,
-                    detail::section::overlaps_section_box
-                >::apply(monotonic_sections, visitor);
+                    robust_box_type
+                >::apply(monotonic_sections, visitor,
+                         detail::section::get_section_box(),
+                         detail::section::overlaps_section_box());
         }
 
         insert_rescaled_piece_turns();
@@ -720,17 +920,17 @@ struct buffered_piece_collection
 
             geometry::partition
                 <
-                    robust_box_type,
-                    turn_get_box, turn_ovelaps_box,
-                    piece_get_box, piece_ovelaps_box
-                >::apply(m_turns, m_pieces, visitor);
+                    robust_box_type
+                >::apply(m_turns, m_pieces, visitor,
+                         turn_get_box(), turn_ovelaps_box(),
+                         piece_get_box(), piece_ovelaps_box());
 
         }
     }
 
-    inline void start_new_ring()
+    inline void start_new_ring(bool deflate)
     {
-        int const n = offsetted_rings.size();
+        signed_size_type const n = static_cast<signed_size_type>(offsetted_rings.size());
         current_segment_id.source_index = 0;
         current_segment_id.multi_index = n;
         current_segment_id.ring_index = -1;
@@ -739,12 +939,42 @@ struct buffered_piece_collection
         offsetted_rings.resize(n + 1);
         current_robust_ring.clear();
 
-        m_first_piece_index = boost::size(m_pieces);
+        m_first_piece_index = static_cast<signed_size_type>(boost::size(m_pieces));
+        m_deflate = deflate;
+        if (deflate)
+        {
+            // Pieces contain either deflated exterior rings, or inflated
+            // interior rings which are effectively deflated too
+            m_has_deflated = true;
+        }
+    }
+
+    inline void abort_ring()
+    {
+        // Remove all created pieces for this ring, sections, last offsetted
+        while (! m_pieces.empty()
+               && m_pieces.back().first_seg_id.multi_index
+               == current_segment_id.multi_index)
+        {
+            m_pieces.erase(m_pieces.end() - 1);
+        }
+
+        while (! monotonic_sections.empty()
+               && monotonic_sections.back().ring_id.multi_index
+               == current_segment_id.multi_index)
+        {
+            monotonic_sections.erase(monotonic_sections.end() - 1);
+        }
+
+        offsetted_rings.erase(offsetted_rings.end() - 1);
+        current_robust_ring.clear();
+
+        m_first_piece_index = -1;
     }
 
     inline void update_closing_point()
     {
-        BOOST_ASSERT(! offsetted_rings.empty());
+        BOOST_GEOMETRY_ASSERT(! offsetted_rings.empty());
         buffered_ring<Ring>& added = offsetted_rings.back();
         if (! boost::empty(added))
         {
@@ -764,7 +994,7 @@ struct buffered_piece_collection
         // For now, it is made equal because due to numerical instability,
         // it can be a tiny bit off, possibly causing a self-intersection
 
-        BOOST_ASSERT(boost::size(m_pieces) > 0);
+        BOOST_GEOMETRY_ASSERT(boost::size(m_pieces) > 0);
         if (! ring.empty()
             && current_segment_id.segment_index
                 == m_pieces.back().first_seg_id.segment_index)
@@ -773,19 +1003,33 @@ struct buffered_piece_collection
         }
     }
 
-    inline void finish_ring(bool is_interior = false, bool has_interiors = false)
+    inline void set_piece_center(point_type const& center)
     {
+        BOOST_GEOMETRY_ASSERT(! m_pieces.empty());
+        geometry::recalculate(m_pieces.back().robust_center, center,
+                m_robust_policy);
+    }
+
+    inline void finish_ring(strategy::buffer::result_code code,
+                            bool is_interior = false, bool has_interiors = false)
+    {
+        if (code == strategy::buffer::result_error_numerical)
+        {
+            abort_ring();
+            return;
+        }
+
         if (m_first_piece_index == -1)
         {
             return;
         }
 
-        if (m_first_piece_index < static_cast<int>(boost::size(m_pieces)))
+        if (m_first_piece_index < static_cast<signed_size_type>(boost::size(m_pieces)))
         {
             // If piece was added
             // Reassign left-of-first and right-of-last
             geometry::range::at(m_pieces, m_first_piece_index).left_index
-                                                    = boost::size(m_pieces) - 1;
+                    = static_cast<signed_size_type>(boost::size(m_pieces)) - 1;
             geometry::range::back(m_pieces).right_index = m_first_piece_index;
         }
         m_first_piece_index = -1;
@@ -794,7 +1038,7 @@ struct buffered_piece_collection
 
         if (! current_robust_ring.empty())
         {
-            BOOST_ASSERT
+            BOOST_GEOMETRY_ASSERT
             (
                 geometry::equals(current_robust_ring.front(),
                     current_robust_ring.back())
@@ -808,20 +1052,20 @@ struct buffered_piece_collection
 
     inline void set_current_ring_concave()
     {
-        BOOST_ASSERT(boost::size(offsetted_rings) > 0);
+        BOOST_GEOMETRY_ASSERT(boost::size(offsetted_rings) > 0);
         offsetted_rings.back().has_concave = true;
     }
 
-    inline int add_point(point_type const& p)
+    inline signed_size_type add_point(point_type const& p)
     {
-        BOOST_ASSERT(boost::size(offsetted_rings) > 0);
+        BOOST_GEOMETRY_ASSERT(boost::size(offsetted_rings) > 0);
 
         buffered_ring<Ring>& current_ring = offsetted_rings.back();
         update_last_point(p, current_ring);
 
         current_segment_id.segment_index++;
         current_ring.push_back(p);
-        return current_ring.size();
+        return static_cast<signed_size_type>(current_ring.size());
     }
 
     //-------------------------------------------------------------------------
@@ -836,7 +1080,11 @@ struct buffered_piece_collection
 
         piece pc;
         pc.type = type;
-        pc.index = boost::size(m_pieces);
+        pc.index = static_cast<signed_size_type>(boost::size(m_pieces));
+        pc.is_deflated = m_deflate;
+
+        current_segment_id.piece_index = pc.index;
+
         pc.first_seg_id = current_segment_id;
 
         // Assign left/right (for first/last piece per ring they will be re-assigned later)
@@ -862,11 +1110,11 @@ struct buffered_piece_collection
             return;
         }
 
-        BOOST_ASSERT(pc.first_seg_id.multi_index >= 0);
-        BOOST_ASSERT(pc.last_segment_index >= 0);
+        BOOST_GEOMETRY_ASSERT(pc.first_seg_id.multi_index >= 0);
+        BOOST_GEOMETRY_ASSERT(pc.last_segment_index >= 0);
 
         pc.offsetted_count = pc.last_segment_index - pc.first_seg_id.segment_index;
-        BOOST_ASSERT(pc.offsetted_count >= 0);
+        BOOST_GEOMETRY_ASSERT(pc.offsetted_count >= 0);
 
         pc.robust_ring.reserve(pc.offsetted_count + helper_points_size);
 
@@ -915,11 +1163,10 @@ struct buffered_piece_collection
             return;
         }
 
-        geometry::detail::envelope::envelope_range::apply(pc.robust_ring,
-                pc.robust_envelope);
+        geometry::envelope(pc.robust_ring, pc.robust_envelope);
 
         geometry::assign_inverse(pc.robust_offsetted_envelope);
-        for (int i = 0; i < pc.offsetted_count; i++)
+        for (signed_size_type i = 0; i < pc.offsetted_count; i++)
         {
             geometry::expand(pc.robust_offsetted_envelope, pc.robust_ring[i]);
         }
@@ -1010,7 +1257,7 @@ struct buffered_piece_collection
     template <typename Range>
     inline void add_range_to_piece(piece& pc, Range const& range, bool add_front)
     {
-        BOOST_ASSERT(boost::size(range) != 0u);
+        BOOST_GEOMETRY_ASSERT(boost::size(range) != 0u);
 
         typename Range::const_iterator it = boost::begin(range);
 
@@ -1046,7 +1293,7 @@ struct buffered_piece_collection
     inline void add_side_piece(point_type const& p1, point_type const& p2,
             Range const& range, bool first)
     {
-        BOOST_ASSERT(boost::size(range) >= 2u);
+        BOOST_GEOMETRY_ASSERT(boost::size(range) >= 2u);
 
         piece& pc = create_piece(strategy::buffer::buffered_segment, ! first);
         add_range_to_piece(pc, range, first);
@@ -1093,19 +1340,31 @@ struct buffered_piece_collection
         }
     }
 
+    inline void mark_flat_start()
+    {
+        if (! m_pieces.empty())
+        {
+            piece& back = m_pieces.back();
+            back.is_flat_start = true;
+        }
+    }
+
+    inline void mark_flat_end()
+    {
+        if (! m_pieces.empty())
+        {
+            piece& back = m_pieces.back();
+            back.is_flat_end = true;
+        }
+    }
+
     //-------------------------------------------------------------------------
 
     inline void enrich()
     {
-        typedef typename strategy::side::services::default_strategy
-        <
-            typename cs_tag<Ring>::type
-        >::type side_strategy_type;
-
-        enrich_intersection_points<false, false>(m_turns,
-                    detail::overlay::operation_union,
-                    offsetted_rings, offsetted_rings,
-                    m_robust_policy, side_strategy_type());
+        enrich_intersection_points<false, false, overlay_buffer>(m_turns,
+                    m_clusters, offsetted_rings, offsetted_rings,
+                    m_robust_policy, m_side_strategy);
     }
 
     // Discards all rings which do have not-OK intersection points only.
@@ -1120,7 +1379,7 @@ struct buffered_piece_collection
                 offsetted_rings[it->operations[0].seg_id.multi_index].has_discarded_intersections = true;
                 offsetted_rings[it->operations[1].seg_id.multi_index].has_discarded_intersections = true;
             }
-            else if (! it->both(detail::overlay::operation_union))
+            else
             {
                 offsetted_rings[it->operations[0].seg_id.multi_index].has_accepted_intersections = true;
                 offsetted_rings[it->operations[1].seg_id.multi_index].has_accepted_intersections = true;
@@ -1133,7 +1392,7 @@ struct buffered_piece_collection
         robust_point_type any_point;
         geometry::recalculate(any_point, point, m_robust_policy);
 
-        int count_in_original = 0;
+        signed_size_type count_in_original = 0;
 
         // Check of the robust point of this outputted ring is in
         // any of the robust original rings
@@ -1192,7 +1451,7 @@ struct buffered_piece_collection
             buffered_ring<Ring>& ring = *it;
             if (! ring.has_intersections()
                 && boost::size(ring) > 0u
-                && geometry::area(ring) < 0)
+                && geometry::area(ring, m_area_strategy) < 0)
             {
                 if (! point_coveredby_original(geometry::range::front(ring)))
                 {
@@ -1216,17 +1475,12 @@ struct buffered_piece_collection
         for (typename boost::range_iterator<turn_vector_type>::type it =
             boost::begin(m_turns); it != boost::end(m_turns); ++it)
         {
-            if (it->location != location_ok)
+            buffer_turn_info_type& turn = *it;
+            if (turn.location != location_ok)
             {
-                // Set it to blocked. They should not be discarded, to avoid
-                // generating rings over these turns
-                // Performance goes down a tiny bit from 161 s to 173 because there
-                // are sometimes much more turns.
-                // We might speed it up a bit by keeping only one blocked
-                // intersection per segment, but that is complex to program
-                // because each turn involves two segments
-                it->operations[0].operation = detail::overlay::operation_blocked;
-                it->operations[1].operation = detail::overlay::operation_blocked;
+                // Discard this turn (don't set it to blocked to avoid colocated
+                // clusters being discarded afterwards
+                turn.discarded = true;
             }
         }
     }
@@ -1238,13 +1492,18 @@ struct buffered_piece_collection
                 false, false,
                 buffered_ring_collection<buffered_ring<Ring> >,
                 buffered_ring_collection<buffered_ring<Ring > >,
+                overlay_buffer,
                 backtrack_for_buffer
             > traverser;
+        std::map<ring_identifier, overlay::ring_turn_info> turn_info_per_ring;
 
         traversed_rings.clear();
+        buffer_overlay_visitor visitor;
         traverser::apply(offsetted_rings, offsetted_rings,
-                        detail::overlay::operation_union,
-                        m_robust_policy, m_turns, traversed_rings);
+                        m_intersection_strategy, m_robust_policy,
+                        m_turns, traversed_rings,
+                        turn_info_per_ring,
+                        m_clusters, visitor);
     }
 
     inline void reverse()
@@ -1271,7 +1530,7 @@ struct buffered_piece_collection
     template <typename GeometryOutput, typename OutputIterator>
     inline OutputIterator assign(OutputIterator out) const
     {
-        typedef detail::overlay::ring_properties<point_type> properties;
+        typedef detail::overlay::ring_properties<point_type, area_result_type> properties;
 
         std::map<ring_identifier, properties> selected;
 
@@ -1279,7 +1538,7 @@ struct buffered_piece_collection
         // Inner rings, for deflate, which do not have intersections, and
         // which are outside originals, are skipped
         // (other ones should be traversed)
-        int index = 0;
+        signed_size_type index = 0;
         for(typename buffered_ring_collection<buffered_ring<Ring> >::const_iterator it = boost::begin(offsetted_rings);
             it != boost::end(offsetted_rings);
             ++it, ++index)
@@ -1287,8 +1546,12 @@ struct buffered_piece_collection
             if (! it->has_intersections()
                 && ! it->is_untouched_outside_original)
             {
-                ring_identifier id(0, index, -1);
-                selected[id] = properties(*it);
+                properties p = properties(*it, m_area_strategy);
+                if (p.valid)
+                {
+                    ring_identifier id(0, index, -1);
+                    selected[id] = p;
+                }
             }
         }
 
@@ -1299,12 +1562,20 @@ struct buffered_piece_collection
                 it != boost::end(traversed_rings);
                 ++it, ++index)
         {
-            ring_identifier id(2, index, -1);
-            selected[id] = properties(*it);
+            properties p = properties(*it, m_area_strategy);
+            if (p.valid)
+            {
+                ring_identifier id(2, index, -1);
+                selected[id] = p;
+            }
         }
 
-        detail::overlay::assign_parents(offsetted_rings, traversed_rings, selected, true);
-        return detail::overlay::add_rings<GeometryOutput>(selected, offsetted_rings, traversed_rings, out);
+        // Assign parents, checking orientation but NOT discarding double
+        // negative rings (negative child with negative parent)
+        detail::overlay::assign_parents(offsetted_rings, traversed_rings,
+                selected, m_intersection_strategy, true, false);
+        return detail::overlay::add_rings<GeometryOutput>(selected, offsetted_rings, traversed_rings, out,
+                                                          m_area_strategy);
     }
 
 };
