@@ -6,6 +6,7 @@
 # define ITERATOR_DWA2002510_HPP
 
 # include <boost/python/detail/prefix.hpp>
+# include <boost/python/detail/type_traits.hpp>
 
 # include <boost/python/class.hpp>
 # include <boost/python/return_value_policy.hpp>
@@ -24,10 +25,6 @@
 
 # include <boost/type.hpp>
 
-# include <boost/type_traits/is_same.hpp>
-# include <boost/type_traits/add_reference.hpp>
-# include <boost/type_traits/add_const.hpp>
-
 # include <boost/detail/iterator.hpp>
 
 namespace boost { namespace python { namespace objects {
@@ -71,35 +68,7 @@ struct iterator_range
 # endif 
     };
     
-# ifdef BOOST_NO_TEMPLATE_PARTIAL_SPECIALIZATION
-    // for compilers which can't deduce the value_type of pointers, we
-    // have a special implementation of next.  This takes advantage of
-    // the fact that T* results are treated like T& results by
-    // Boost.Python's function wrappers.
-    struct next_ptr
-    {
-        typedef Iterator result_type;
-        
-        result_type
-        operator()(iterator_range<NextPolicies,Iterator>& self)
-        {
-            if (self.m_start == self.m_finish)
-                stop_iteration_error();
-            return self.m_start++;
-        }
-    };
-    
-    typedef mpl::if_<
-        is_same<
-            boost::detail::please_invoke_BOOST_TT_BROKEN_COMPILER_SPEC_on_cv_unqualified_pointee<Iterator>
-          , typename traits_t::value_type
-        >
-      , next_ptr
-      , next
-    >::type next_fn;
-# else
     typedef next next_fn;
-# endif
     
     object m_sequence; // Keeps the sequence alive while iterating.
     Iterator m_start;
@@ -129,7 +98,11 @@ namespace detail
       return class_<range_>(name, no_init)
           .def("__iter__", identity_function())
           .def(
+#if PY_VERSION_HEX >= 0x03000000
+              "__next__"
+#else
               "next"
+#endif
             , make_function(
                 next_fn()
               , policies
@@ -226,8 +199,8 @@ inline object make_iterator_function(
 )
 {
     typedef typename Accessor1::result_type iterator;
-    typedef typename add_const<iterator>::type iterator_const;
-    typedef typename add_reference<iterator_const>::type iterator_cref;
+    typedef typename boost::python::detail::add_const<iterator>::type iterator_const;
+    typedef typename boost::python::detail::add_lvalue_reference<iterator_const>::type iterator_cref;
       
     return detail::make_iterator_function(
         get_start
